@@ -6,12 +6,13 @@ import requests
 import shutil
 import pathlib
 import zipfile
+import queue
 
 from .State import _State
 from .Base import _Base, _check_for_error
 from .Datasource import _Datasource
 
-RETRY_AFTER = 5 # seconds
+RETRY_AFTER = 5  # seconds
 
 class _MessageHandler(_Base):
 
@@ -21,6 +22,8 @@ class _MessageHandler(_Base):
         self.wshost = self.host.replace('http', 'ws', 1)
         self.state: _State = state
 
+        self.task_queue = queue.Queue()
+
         self.ws = websocket.WebSocketApp(
             self.wshost,
             on_open=self._on_open,
@@ -37,18 +40,30 @@ class _MessageHandler(_Base):
             'hyperparams-updated': self.hyperparams_updated,
             'dynamic-trainer': self.dynamic_trainer,
             'round-error': self.round_error,
-
+            'startup-snapshot': self.startup_snapshot,
             'feature-forward': self.feature_forward,
             'feature-backward': self.feature_backward,
             'label-train': self.label_train,
             'label-test': self.label_test
         }
+        self.worker_handlers = ['client-info', 'new-project', 'round-complete', 'dynamic-trainer',
+                                'startup-snapshot', 'feature-forward', 'feature-backward', 'label-train', 'label-test']
 
     def connect_to_ws(self):
         t = threading.Thread(target=self.ws.run_forever, kwargs={'reconnect': RETRY_AFTER})
         t.daemon = False
         t.start()
 
+        worker_t = threading.Thread(target=self._worker)
+        worker_t.daemon = True
+        worker_t.start()
+
+    def _worker(self):
+        while True:
+            task = self.task_queue.get()
+            self.handlers[task['event']](task['j'], task['project_key'])
+            self.task_queue.task_done()
+
     def _on_open(self, ws):
         print('>>> Connection to DynamoFL established.')
         self.ws.send('{ "action": "auth", "token": "' + self.token + '" }')
@@ -61,7 +76,10 @@ class _MessageHandler(_Base):
             project_key = j['data']['project']['key']
 
         if j['event'] in self.handlers:
-            self.handlers[j['event']](j, project_key)
+            if j['event'] in self.worker_handlers:
+                self.task_queue.put_nowait({'event': j['event'], 'j': j, 'project_key': project_key})
+            else:
+                self.handlers[j['event']](j, project_key)
 
     def _on_close(self, ws, close_status_code, close_msg):
         print('Connection closed')
@@ -109,7 +127,7 @@ class _MessageHandler(_Base):
             info = self._make_request('GET', f'/projects/{project_key}')
             self.state.train_and_test_callback(datasource_key, info)
             return
-        
+
         vertical_participant = {
             'project_key': project_key,
             'datasource_key': datasource_key,
@@ -299,8 +317,15 @@ class _MessageHandler(_Base):
         for p in self.state.project_participants:
             print('Federation error occured:\n  ' + j['data']['errorMessage'])
 
-def get_threads_key(project_key, datasource_key):
-    return project_key + '.' + datasource_key
+    def startup_snapshot(self, j, _):
+        datasource_key = j['data']['datasourceKey']
+        snapshot = j['data']['snapshot']
+        
+        for p in snapshot['projects']:
+            if (p['currentRound'] == p['rounds']):
+                return
+
+            self.state.train_and_test_callback(datasource_key, p)
 
 def get_vertical_file_path(type, project_key, ds, round):
     return f'{type}_{project_key}_{ds}_{round}.any'
