@@ -114,6 +114,15 @@ class _Project(_Base):
                 else:
                     raise e
             
+    """
+        Uploads pre-intersected list of ids to newly created project.
+    """
+    def push_ids(self, base_file):
+        with open(base_file, 'rb') as f:
+            try:
+                self._make_request('POST', f'/projects/{self.key}/ids', files={'ids': f})
+            except Exception as e:
+                print('Something went wrong')
 
     def pull_model(self, filepath, datasource_key=None, round=None, federated_model=None):
         params = {
