@@ -1,5 +1,6 @@
 from .State import _State
 from .MessageHandler import _MessageHandler
+from .helpers import _Helpers
 
 RETRY_AFTER = 5 # seconds
 
@@ -7,6 +8,8 @@ class DynamoFL():
     def __init__(self, token, host='https://api.dynamofl.com', metadata=None):
         self._state = _State(token, host, metadata)
 
+        self._helpers = _Helpers()
+
         self._messagehandler = _MessageHandler(self._state, token, host)
         self._messagehandler.connect_to_ws()
 
@@ -16,6 +19,9 @@ class DynamoFL():
     def delete_datasource(self, key):
         return self._state.delete_datasource(key)
 
+    def get_datasources(self):
+        return self._state.get_datasources()
+
     def delete_project(self, key):
         return self._state.delete_project(key)
 
@@ -30,11 +36,17 @@ class DynamoFL():
 
     def get_projects(self):
         return self._state.get_project()
+    
+    def load_state_dict_tensorflow(self, model):
+        return self._helpers.load_state_dict_tensorflow(model)
+    
+    def save_state_dict_tensorflow(self, model, state_dict):
+        return self._helpers.save_state_dict_tensorflow(model, state_dict)
 
     def is_datasource_labeled(self, project_key=None, datasource_key=None):
-        """
+        '''
         Accepts a valid datasource_key and project_key
         Returns True if the datasource is labeled for the project; False otherwise
         
-        """
-        return self._state.is_datasource_labeled(project_key=project_key, datasource_key=datasource_key)
\ No newline at end of file
+        '''
+        return self._state.is_datasource_labeled(project_key=project_key, datasource_key=datasource_key)
